# Generated by Django 3.2.11 on 2022-07-22 19:12
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from edc_egfr.calculators import EgfrCkdEpi, egfr_percent_change
from edc_visit_schedule.constants import DAY1
from tqdm import tqdm


def update_revised_egrf(apps, schema_editor):
    bloodresultrft_model_cls = apps.get_model("meta_subject.bloodresultsrft")
    subjectvisit_model_cls = apps.get_model("meta_subject.subjectvisit")
    try:
        subjectscreening_model_cls = apps.get_model("meta_screening.subjectscreening")
    except LookupError:
        pass
    else:
        total = bloodresultrft_model_cls.objects.all().count()
        for obj in tqdm(bloodresultrft_model_cls.objects.all(), total=total):
            subject_visit = subjectvisit_model_cls.objects.get(id=obj.subject_visit_id)
            subject_screening = subjectscreening_model_cls.objects.get(
                subject_identifier=subject_visit.subject_identifier
            )
            try:
                baseline_obj = bloodresultrft_model_cls.objects.get(
                    subject_visit__subject_identifier=subject_visit.subject_identifier,
                    subject_visit__visit_code=DAY1,
                    subject_visit__visit_code_sequence=0,
                )
            except ObjectDoesNotExist:
                pass
            else:
                obj.old_egfr_value = obj.egfr_value
                obj.old_egfr_drop_value = obj.egfr_drop_value
                obj.egfr_value = EgfrCkdEpi(
                    gender=subject_screening.gender,
                    ethnicity=subject_screening.ethnicity,
                    age_in_years=subject_screening.age_in_years,
                    creatinine_value=obj.creatinine_value,
                    creatinine_units=obj.creatinine_units,
                ).value
                obj.egfr_drop_value = egfr_percent_change(
                    float(obj.egfr_value), float(baseline_obj.egfr_value)
                )
                obj.save_base(
                    update_fields=[
                        "egfr_value",
                        "egfr_drop_value",
                        "old_egfr_value",
                        "old_egfr_drop_value",
                    ]
                )


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0134_auto_20220722_2211"),
    ]

    operations = [migrations.RunPython(update_revised_egrf)]



from __future__ import annotations

from decimal import Decimal
from typing import Type

from django.contrib import admin, messages
from django.contrib.admin import SimpleListFilter
from django.contrib.messages import ERROR, SUCCESS
from django.core.exceptions import ObjectDoesNotExist
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_model_admin.history import SimpleHistoryAdmin
from edc_offstudy.exceptions import OffstudyError
from edc_pharmacy.exceptions import NextStudyMedicationError, RefillCreatorError
from edc_pharmacy.model_mixins.study_medication_crf_model_mixin import (
    StudyMedicationError,
)
from edc_pharmacy.models import DosageGuideline, Formulation, Medication
from edc_utils.date import to_local
from edc_visit_schedule.utils import is_baseline

from meta_pharmacy.constants import METFORMIN

from ..admin_site import meta_subject_admin
from ..forms import StudyMedicationForm
from ..models import StudyMedication
from .list_filters import RefillEndDateListFilter, RefillStartDateListFilter
from .modeladmin import CrfModelAdminMixin


@admin.register(StudyMedication, site=meta_subject_admin)
class StudyMedicationAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):

    actions = ["create_or_update_rx_refills"]

    form = StudyMedicationForm

    autocomplete_fields = [
        "dosage_guideline",
        "formulation",
    ]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                    "refill",
                )
            },
        ),
        (
            "Details",
            {
                "fields": (
                    "refill_start_datetime",
                    "dosage_guideline",
                    "formulation",
                    "refill_to_next_visit",
                    "refill_end_datetime",
                    "special_instructions",
                ),
            },
        ),
        (
            "Dispensed medication",
            {
                "description": (
                    "Type in the stock code on the medication "
                    "bottle or bottles. Separate by comma."
                ),
                "fields": ("stock_codes",),
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "formulation": admin.VERTICAL,
        "refill": admin.VERTICAL,
        "refill_to_next_visit": admin.VERTICAL,
    }

    def get_list_filter(self, request) -> tuple[str | Type[SimpleListFilter], ...]:
        list_filter = super().get_list_filter(request)
        list_filter = list(list_filter)
        list_filter.insert(4, "dosage_guideline")
        list_filter.insert(4, RefillEndDateListFilter)
        list_filter.insert(4, RefillStartDateListFilter)
        list_filter = tuple(list_filter)
        return list_filter

    def get_list_display(self, request) -> tuple[str, ...]:
        list_display = super().get_list_display(request)
        list_display = list(list_display)
        list_display.insert(4, "refill_days")
        list_display.insert(4, "refill_end_date")
        list_display.insert(4, "refill_start_date")
        list_display.insert(4, "dosage_guideline")
        list_display = tuple(list_display)
        return list_display

    @admin.display(description="days", ordering="number_of_days")
    def refill_days(self, obj):
        return obj.number_of_days

    @admin.display(description="refill_start", ordering="refill_start_datetime")
    def refill_start_date(self, obj):
        if obj.refill_start_datetime:
            return to_local(obj.refill_start_datetime).date()
        return None

    @admin.display(description="refill_end", ordering="refill_end_datetime")
    def refill_end_date(self, obj):
        if obj.refill_end_datetime:
            return to_local(obj.refill_end_datetime).date()
        return None

    @admin.action(permissions=["view"], description="Create or update RX Refills")
    def create_or_update_rx_refills(self, request, queryset):
        updated = 0
        errors = 0
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.get(medication=medication, strength=500)
        dosage_guideline_baseline = DosageGuideline.objects.get(
            medication=medication, dose=Decimal("1000.0")
        )
        dosage_guideline_followup = DosageGuideline.objects.get(
            medication=medication, dose=Decimal("2000.0")
        )
        for obj in queryset:
            obj.refill_identifier = obj.id
            if not obj.formulation:
                obj.formulation = formulation
            if not obj.dosage_guideline:
                if is_baseline(obj.related_visit):
                    obj.dosage_guideline = dosage_guideline_baseline
                else:
                    obj.dosage_guideline = dosage_guideline_followup
            updated, errors = self.rx_refills_save_or_raise(request, obj, updated, errors)
        messages.add_message(
            request, SUCCESS, f"Updated {updated} documents with {errors} errors."
        )

    @staticmethod
    def rx_refills_save_or_raise(request, obj, updated, errors):
        try:
            obj.save()
        except NextStudyMedicationError as e:
            messages.add_message(request, ERROR, f"NextStudyMedicationError: {e}")
            errors += 1
        except ObjectDoesNotExist as e:
            messages.add_message(
                request,
                ERROR,
                (
                    f"ObjectDoesNotExist: {obj.subject_identifier}, "
                    f"{obj.related_visit}, {str(e)}"
                ),
            )
            errors += 1
        except StudyMedicationError as e:
            messages.add_message(request, ERROR, f"StudyMedicationError: {e}")
            errors += 1
        except RefillCreatorError as e:
            messages.add_message(request, ERROR, f"RefillCreatorError: {e}")
            errors += 1
        except OffstudyError as e:
            messages.add_message(request, ERROR, f"OffstudyError: {e}")
            errors += 1
        else:
            updated += 1
        return updated, errors



# Generated by Django 4.2.11 on 2024-05-15 17:56

import _socket
from django.conf import settings
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import simple_history.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("meta_subject", "0187_dmdiagnosis_historicaldmdiagnosis_dmdxresult_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalDmDxResult",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("report_date", models.DateField(verbose_name="Date")),
                (
                    "utestid",
                    models.CharField(
                        choices=[("fbg", "FBG")], max_length=15, verbose_name="Test"
                    ),
                ),
                (
                    "quantifier",
                    models.CharField(
                        choices=[
                            ("=", "="),
                            (">", ">"),
                            (">=", ">="),
                            ("<", "<"),
                            ("<=", "<="),
                        ],
                        default="=",
                        max_length=10,
                    ),
                ),
                ("value", models.DecimalField(decimal_places=2, max_digits=8)),
                (
                    "units",
                    models.CharField(
                        choices=[
                            ("mg/dL", "mg/dL"),
                            ("mmol/L", "mmol/L (millimoles/L)"),
                            ("%", "%"),
                        ],
                        max_length=15,
                        verbose_name="Units",
                    ),
                ),
                (
                    "fasted",
                    models.BooleanField(default=False, max_length=15, verbose_name="Fasted?"),
                ),
                ("comment", models.CharField(blank=True, max_length=35, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "dm_diagnosis",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.dmdiagnosis",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Diabetes diagmnosis: Result",
                "verbose_name_plural": "historical Diabetes diagmnosis: Results",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="DmDxResult",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("report_date", models.DateField(verbose_name="Date")),
                (
                    "utestid",
                    models.CharField(
                        choices=[("fbg", "FBG")], max_length=15, verbose_name="Test"
                    ),
                ),
                (
                    "quantifier",
                    models.CharField(
                        choices=[
                            ("=", "="),
                            (">", ">"),
                            (">=", ">="),
                            ("<", "<"),
                            ("<=", "<="),
                        ],
                        default="=",
                        max_length=10,
                    ),
                ),
                ("value", models.DecimalField(decimal_places=2, max_digits=8)),
                (
                    "units",
                    models.CharField(
                        choices=[
                            ("mg/dL", "mg/dL"),
                            ("mmol/L", "mmol/L (millimoles/L)"),
                            ("%", "%"),
                        ],
                        max_length=15,
                        verbose_name="Units",
                    ),
                ),
                (
                    "fasted",
                    models.BooleanField(default=False, max_length=15, verbose_name="Fasted?"),
                ),
                ("comment", models.CharField(blank=True, max_length=35, null=True)),
                (
                    "dm_diagnosis",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.dmdiagnosis",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Diabetes diagmnosis: Result",
                "verbose_name_plural": "Diabetes diagmnosis: Results",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["modified", "created"], name="meta_subjec_modifie_6c33f9_idx"
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="meta_subjec_user_mo_a4a0c6_idx",
                    ),
                ],
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



# Generated by Django 4.2.11 on 2024-07-27 00:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0006_endpoints_baseline_datetime"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endpoints",
            name="endpoint_label",
            field=models.CharField(max_length=25, null=True),
        ),
        migrations.AlterField(
            model_name="endpoints",
            name="fasting",
            field=models.CharField(max_length=10, null=True),
        ),
        migrations.AlterField(
            model_name="endpoints",
            name="fbg_datetime",
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name="endpoints",
            name="fbg_value",
            field=models.FloatField(null=True),
        ),
        migrations.AlterField(
            model_name="endpoints",
            name="ogtt_value",
            field=models.FloatField(null=True),
        ),
        migrations.AlterField(
            model_name="endpoints",
            name="visit_code",
            field=models.IntegerField(null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-16 00:52
from django.db import migrations
from edc_model.utils import duration_hm_to_timedelta
from tqdm import tqdm


def update_duration(apps, schema_editor):
    model_cls = apps.get_model("meta_screening.subjectscreening")
    qs = model_cls.objects.all()
    total = qs.count()
    for obj in tqdm(qs, total=total):
        if obj.fasting_duration_str:
            tdelta = duration_hm_to_timedelta(obj.fasting_duration_str)
            obj.fasting_duration_delta = tdelta
            obj.save_base(update_fields=["fasting_duration_delta"])
        if obj.repeat_fasting_duration_str:
            tdelta = duration_hm_to_timedelta(obj.repeat_fasting_duration_str)
            obj.repeat_fasting_duration_delta = tdelta
            obj.save_base(update_fields=["repeat_fasting_duration_delta"])


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_screening",
            "0064_remove_historicalscreeningpartone_fasting_duration_minutes_and_more",
        ),
    ]

    operations = [migrations.RunPython(update_duration)]



# Generated by Django 3.2.11 on 2022-03-07 16:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_ae", "0008_auto_20211011_1657"),
    ]

    operations = [
        migrations.AlterField(
            model_name="deathreport",
            name="death_location",
            field=models.CharField(
                choices=[
                    ("home", "At home"),
                    ("hospital_clinic", "Hospital/clinic"),
                    ("elsewhere", "Elsewhere"),
                ],
                max_length=50,
                verbose_name="Where did the participant die?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="death_location",
            field=models.CharField(
                choices=[
                    ("home", "At home"),
                    ("hospital_clinic", "Hospital/clinic"),
                    ("elsewhere", "Elsewhere"),
                ],
                max_length=50,
                verbose_name="Where did the participant die?",
            ),
        ),
    ]



from .draw_label_for_subject_with_code128 import draw_label_for_subject_with_code128
from .draw_label_with_test_data import draw_label_with_test_data

# from .label_data import LabelData
# from .print_sheets import print_sheets



import numpy as np
import pandas as pd
from django_pandas.io import read_frame
from edc_pdutils.dataframes import get_eos, get_subject_consent, get_subject_visit

from meta_subject.models import Glucose, GlucoseFbg


def get_glucose_df() -> pd.DataFrame:
    qs_glucose_fbg = GlucoseFbg.objects.all()
    df_glucose_fbg = read_frame(qs_glucose_fbg)
    df_glucose_fbg.rename(
        columns={"fbg_fasting": "fasting", "subject_visit": "subject_visit_id"},
        inplace=True,
    )

    df_glucose_fbg.loc[(df_glucose_fbg["fasting"] == "fasting"), "fasting"] = "Yes"
    df_glucose_fbg.loc[(df_glucose_fbg["fasting"] == "non_fasting"), "fasting"] = "No"
    df_glucose_fbg["fasting_hrs"] = np.nan
    df_glucose_fbg["fasting_hrs"] = df_glucose_fbg["fasting_duration_delta"].apply(
        lambda x: x.total_seconds() / 3600
    )
    df_glucose_fbg["fasting_hrs"] = df_glucose_fbg["fasting_hrs"].apply(
        lambda x: 8.05 if not x else x
    )
    # df_glucose_fbg = df_glucose_fbg.loc[df_glucose_fbg["fasting_hrs"] >= 8.0]
    # df_glucose_fbg.reset_index(drop=True, inplace=True)
    df_glucose_fbg.loc[
        :,
        ["ogtt_value", "ogtt_units", "ogtt_datetime"],
    ] = [np.nan, None, pd.NaT]
    df_glucose_fbg["source"] = "meta_subject.glucosefbg"

    qs_glucose = Glucose.objects.all()
    df_glucose = read_frame(qs_glucose)
    df_glucose.rename(columns={"subject_visit": "subject_visit_id"}, inplace=True)
    df_glucose.loc[(df_glucose["fasting"] == "fasting"), "fasting"] = "Yes"
    df_glucose.loc[(df_glucose["fasting"] == "non_fasting"), "fasting"] = "No"
    df_glucose["fasting_hrs"] = np.nan
    df_glucose["fasting_hrs"] = df_glucose[df_glucose["fasting"] == "Yes"][
        "fasting_duration_delta"
    ].apply(lambda x: x.total_seconds() / 3600)
    df_glucose["fasting_hrs"] = df_glucose["fasting_hrs"].apply(lambda x: 8.05 if not x else x)
    # df_glucose = df_glucose.loc[df_glucose["fasting_hrs"] >= 8.0]
    # df_glucose.reset_index(drop=True, inplace=True)
    df_glucose["source"] = "meta_subject.glucose"

    keep_cols = [
        "subject_visit_id",
        "fasting",
        "fasting_hrs",
        "fbg_value",
        "fbg_units",
        "fbg_datetime",
        "ogtt_value",
        "ogtt_units",
        "ogtt_datetime",
        "source",
        "revision",
        "report_datetime",
    ]
    df_glucose = df_glucose[keep_cols]
    df_glucose_fbg = df_glucose_fbg[keep_cols]
    # df = pd.concat([df_glucose_fbg, df_glucose])
    df = pd.merge(
        df_glucose,
        df_glucose_fbg,
        on="subject_visit_id",
        how="outer",
        indicator=True,
        suffixes=("", "_2"),
    )

    for suffix in ["", "_2"]:
        cols = [f"fasting_hrs{suffix}", f"fbg_value{suffix}", f"ogtt_value{suffix}"]
        df[cols] = df[cols].apply(pd.to_numeric)
        cols = [f"fbg_datetime{suffix}", f"ogtt_datetime{suffix}"]
        df[cols] = df[cols].apply(pd.to_datetime)
        df.loc[
            (df[f"fbg_units{suffix}"] != "mmol/L (millimoles/L)")
            & (df[f"fbg_value{suffix}"] >= 0),
            f"fbg_units{suffix}",
        ] = "mmol/L (millimoles/L)"
        df.loc[
            (df[f"ogtt_units{suffix}"] != "mmol/L (millimoles/L)")
            & (df[f"ogtt_value{suffix}"] >= 0),
            f"ogtt_units{suffix}",
        ] = "mmol/L (millimoles/L)"
        # remove values if not fasted
        # df.loc[(df[f"fasting{suffix}"] != YES), f"fbg_value{suffix}"] = np.nan
        # df.loc[(df[f"fasting{suffix}"] != YES), f"ogtt_value{suffix}"] = np.nan

    # reconcile all to single column
    df.loc[(df["fbg_value"].isna()) & (df["fbg_value_2"].notna()), "fbg_value"] = df[
        "fbg_value_2"
    ]
    df.loc[(df["ogtt_value"].isna()) & (df["ogtt_value_2"].notna()), "ogtt_value"] = df[
        "ogtt_value_2"
    ]
    cols = [col for col in list(df.columns) if col.endswith("_2")]
    df.drop(columns=cols, inplace=True)
    cols = [col for col in list(df.columns) if col.endswith("_3")]
    df.drop(columns=cols, inplace=True)

    df_subject_visit = get_subject_visit("meta_subject.subjectvisit")
    df_consent = get_subject_consent("meta_consent.subjectconsent")
    df_eos = get_eos("meta_prn.endofstudy")

    df = pd.merge(df_subject_visit, df, on="subject_visit_id", how="left")
    df = pd.merge(df, df_consent, on="subject_identifier", how="left")
    df = pd.merge(df, df_eos, on="subject_identifier", how="left")

    df["visit_days"] = df["baseline_datetime"].rsub(df["visit_datetime"]).dt.days
    df["fgb_days"] = df["baseline_datetime"].rsub(df["fbg_datetime"]).dt.days
    df["ogtt_days"] = df["baseline_datetime"].rsub(df["ogtt_datetime"]).dt.days
    df["visit_days"] = pd.to_numeric(df["visit_days"], downcast="integer")
    df["fgb_days"] = pd.to_numeric(df["fgb_days"], downcast="integer")
    df["ogtt_days"] = pd.to_numeric(df["ogtt_days"], downcast="integer")

    df = df.sort_values(by=["subject_identifier", "visit_code"])
    df.reset_index(drop=True, inplace=True)
    return df



from django.contrib import admin, messages
from django.core.exceptions import FieldDoesNotExist
from django.db import models
from django.db.models import QuerySet
from django.utils.html import format_html
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.list_filters import PastDateListFilter
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_pdutils.actions import export_to_csv
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_sites.admin.list_filters import SiteListFilter
from edc_utils import get_utcnow

from meta_analytics.dataframes import get_last_imp_visits_df

from ..admin_site import meta_reports_admin
from ..models import LastImpRefill


class ImpVisitDateListFilter(PastDateListFilter):
    title = "IMP visit date"

    parameter_name = "imp_visit_date"
    field_name = "imp_visit_date"


class NextApptDateListFilter(PastDateListFilter):
    title = "Next appt date"

    parameter_name = "next_appt_date"
    field_name = "next_appt_date"


def update_report(modeladmin, request, queryset):
    now = get_utcnow()
    modeladmin.model.objects.all().delete()
    df = get_last_imp_visits_df()
    if not df.empty:
        data = [
            modeladmin.model(
                subject_identifier=row["subject_identifier"],
                site_id=row["site_id"],
                imp_visit_date=row["imp_visit_date"],
                imp_visit_code=row["imp_visit_code"],
                next_appt_date=row["next_appt_datetime"],
                next_visit_code=row["next_visit_code"],
                days_since=row["days_since"].days,
                days_until=row["days_until"].days,
                visit_code=str(int(row["imp_visit_code"])),
                visit_code_sequence=row["imp_visit_code"] % 1,
                report_model=modeladmin.model._meta.label_lower,
                created=now,
            )
            for _, row in df.iterrows()
        ]
        created = len(modeladmin.model.objects.bulk_create(data))
        messages.success(request, "{} records were successfully created.".format(created))


@admin.register(LastImpRefill, site=meta_reports_admin)
class LastImpRefillAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    include_note_column = False

    change_list_title = "List of most recent IMP refills per subject"

    change_list_note = format_html(
        """
        This report fetches the most recent Study Medication report where IMP was
        refilled and adds the subject's next visit. Subjects taken "Off Schedule"
        are not included in this report. To update ALL rows in this report, tick
        at least one row and select 'Update report' action below.
        <BR><BR>
        This report has additional search features for numeric columns:
        <code>days_since</code>, <code>days_until</code>, <code>imp_visit_code</code>
        and <code>next_visit_code</code>.
        <BR><BR>For example, type <code>days_until>=25</code> in the search below
        to show rows for subjects who have an appointment 25 or more days from the
        date this report was created. You might also try typing
        <code>days_since>365</code> or <code>days_until<0</code>.
        <BR><BR>
        This also works: <code>next_visit_code>=1060</code>.
        """
    )

    actions = [update_report, export_to_csv]

    list_display = [
        "dashboard",
        "subject_identifier",
        "imp_visit_code",
        "imp_visit_date",
        "days_since",
        "next_visit_code",
        "next_appt_date",
        "days_until",
        "created",
    ]

    list_filter = [
        ImpVisitDateListFilter,
        NextApptDateListFilter,
        "imp_visit_code",
        "next_visit_code",
        SiteListFilter,
    ]

    search_fields = ["subject_identifier"]

    def get_queryset(self, request) -> QuerySet:
        qs = super().get_queryset(request)
        if qs.count() == 0:
            update_report(self, request, None)
            qs = super().get_queryset(request)
        return qs

    def dataframe_to_model(self):
        now = get_utcnow()
        self.model.objects.all().delete()
        created = 0
        df = get_last_imp_visits_df()
        if not df.empty:
            data = [
                self.model(
                    subject_identifier=row["subject_identifier"],
                    site_id=row["site_id"],
                    imp_visit_date=row["imp_visit_date"],
                    imp_visit_code=row["imp_visit_code"],
                    next_appt_date=row["next_appt_datetime"],
                    next_visit_code=row["next_visit_code"],
                    days_since=row["days_since"],
                    days_until=row["days_until"],
                    visit_code=str(int(row["imp_visit_code"])),
                    visit_code_sequence=row["imp_visit_code"] % 1,
                    report_model=self.model._meta.label_lower,
                    created=now,
                )
                for _, row in df.iterrows()
            ]
            created = len(self.model.objects.bulk_create(data))
        return created

    def get_search_results(self, request, queryset, search_term):
        value = None
        fldname = None
        search_term = search_term.replace(" ", "")
        operators = {">=": "gte", "<=": "lte", "<": "lt", ">": "gt"}
        if op := [op for op in operators if op in search_term]:
            op = op[0]
            fldname, value = search_term.split(op)
            try:
                fldcls = self.model._meta.get_field(fldname)
            except FieldDoesNotExist:
                fldname = None
            else:
                if isinstance(fldcls, (models.IntegerField,)):
                    try:
                        value = int(value)
                    except ValueError:
                        value = None
                elif isinstance(fldcls, (models.FloatField,)):
                    try:
                        value = float(value)
                    except ValueError:
                        value = None
        if (
            value is not None
            and fldname
            and fldname in ["days_until", "days_since", "imp_visit_code", "next_visit_code"]
        ):
            queryset, use_distinct = super().get_search_results(request, queryset, None)
            queryset = queryset.filter(**{f"{fldname}__{operators[op]}": value})
        else:
            queryset, use_distinct = super().get_search_results(request, queryset, search_term)
        return queryset, use_distinct



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_action_item.forms import ActionItemFormMixin
from edc_form_validators import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_registration.models import RegisteredSubject
from edc_sites.forms import SiteModelFormMixin

from ..models import SubjectReconsent


class SubjectReconsentForm(
    SiteModelFormMixin,
    ActionItemFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    def clean(self):
        cleaned_data = super().clean()
        try:
            RegisteredSubject.objects.get(
                subject_identifier=cleaned_data.get("subject_identifier"),
                identity=cleaned_data.get("identity"),
            )
        except ObjectDoesNotExist:
            raise forms.ValidationError({"identity": "Identity number does not match."})
        return cleaned_data

    class Meta:
        model = SubjectReconsent
        fields = "__all__"
        help_text = {"action_identifier": "(read-only)", "subject_identifier": "(read-only)"}
        widgets = {
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



# Generated by Django 4.1 on 2022-08-20 00:18

from django.db import migrations, models
import edc_model.validators.date


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0054_auto_20220722_2130"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine &lt;B&gt;level&lt;/B&gt;",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fbg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine &lt;B&gt;level&lt;/B&gt;",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fbg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine &lt;B&gt;level&lt;/B&gt;",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fbg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine &lt;B&gt;level&lt;/B&gt;",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fbg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="creatinine_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine &lt;B&gt;level&lt;/B&gt;",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fbg2_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fbg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="FBG date/time measured"
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt2_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt2_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt2_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_base_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="(2 hours after glucose solution given)",
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date/time blood glucose measured 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose measure 2hrs <u>after</u> oral glucose solution given",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_constants.constants import NONE, YES
from edc_crf.fieldset import crf_status_fieldset
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import FollowupExaminationForm
from ..models import FollowupExamination
from .modeladmin import CrfModelAdminMixin


class GradedEventFilter(admin.SimpleListFilter):
    title = "Graded events"
    parameter_name = "grade"

    def lookups(self, request, model_admin):
        return (
            ("g3", "Grade 3"),
            ("g4", "Grade 4"),
        )

    def queryset(self, request, queryset):
        if self.value() == "g3":
            return queryset.exclude(symptoms_g3__name=NONE)
        if self.value() == "g4":
            return queryset.exclude(symptoms_g4__name=NONE)


@admin.register(FollowupExamination, site=meta_subject_admin)
class FollowupExaminationAdmin(
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = FollowupExaminationForm

    autocomplete_fields = ["art_new_regimen"]

    additional_instructions = [
        "If participant is pregnant, complete the action linked CRF `Pregnancy notification`."
    ]

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Part 1: Symptoms",
            {
                "description": "To be completed by the study physician",
                "fields": (
                    "symptoms",
                    "symptoms_detail",
                    "symptoms_g3",
                    "symptoms_g3_detail",
                    "symptoms_g4",
                    "symptoms_g4_detail",
                ),
            },
        ),
        (
            "Part 2: Other Medical or Health problems",
            {
                "description": "To be completed by the study physician",
                "fields": (
                    "any_other_problems",
                    "any_other_problems_detail",
                    "any_other_problems_sae",
                    "any_other_problems_sae_grade",
                ),
            },
        ),
        (
            "Part 3: Hospitalizations or other external access to care",
            {
                "description": "To be completed by the study physician",
                "fields": (
                    "attended_clinic",
                    "attended_clinic_detail",
                    "admitted_hospital",
                    "attended_clinic_sae",
                    "prescribed_medication",
                    "prescribed_medication_detail",
                ),
            },
        ),
        (
            "Part 4: HIV Medications",
            {
                "description": "To be completed by the study physician",
                "fields": (
                    "art_change",
                    "art_change_reason",
                    "art_new_regimen",
                    "art_new_regimen_other",
                ),
            },
        ),
        (
            "Part 2: Examination",
            {
                "description": "To be completed by the study physician",
                "fields": ("abdominal_tenderness", "enlarged_liver", "jaundice"),
            },
        ),
        (
            "Part 3: Summary",
            {
                "description": "To be completed by the study physician",
                "fields": (
                    "comment",
                    "lactic_acidosis",
                    "hepatomegaly",
                    "referral",
                    "referral_reason",
                ),
            },
        ),
        crf_status_fieldset,
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    filter_horizontal = ("symptoms", "symptoms_g3", "symptoms_g4")

    # readonly_fields = action_fields

    radio_fields = {
        "abdominal_tenderness": admin.VERTICAL,
        "admitted_hospital": admin.VERTICAL,
        "any_other_problems": admin.VERTICAL,
        "any_other_problems_sae": admin.VERTICAL,
        "any_other_problems_sae_grade": admin.VERTICAL,
        "art_change": admin.VERTICAL,
        "art_new_regimen": admin.VERTICAL,
        "attended_clinic": admin.VERTICAL,
        "attended_clinic_sae": admin.VERTICAL,
        "enlarged_liver": admin.VERTICAL,
        "hepatomegaly": admin.VERTICAL,
        "jaundice": admin.VERTICAL,
        "lactic_acidosis": admin.VERTICAL,
        "prescribed_medication": admin.VERTICAL,
        "referral": admin.VERTICAL,
    }

    list_display = ("g3", "g4")

    list_filter = (GradedEventFilter,)

    @staticmethod
    def g3(obj=None):
        if NONE not in [o.name for o in obj.symptoms_g3.all()]:
            return YES
        return None

    @staticmethod
    def g4(obj=None):
        if NONE not in [o.name for o in obj.symptoms_g4.all()]:
            return YES
        return None



# Generated by Django 3.2.4 on 2021-07-21 00:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0006_auto_20210624_0225"),
    ]

    operations = [
        migrations.AddField(
            model_name="endofstudy",
            name="comment",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="Please provide further details if possible",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="comment",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="Please provide further details if possible",
            ),
        ),
    ]



from dateutil.relativedelta import relativedelta
from django.contrib.sites.models import Site
from edc_constants.constants import BLACK, FEMALE, NO, NOT_APPLICABLE, YES
from edc_reportable.units import MILLIGRAMS_PER_DECILITER, MILLIMOLES_PER_LITER
from edc_utils import get_utcnow
from faker import Faker
from model_bakery.recipe import Recipe

from .models import SubjectScreening

fake = Faker()


subjectscreening = Recipe(
    SubjectScreening,
    report_datetime=get_utcnow() - relativedelta(days=1),
    screening_consent=YES,
    hospital_identifier="111",
    initials="ZZ",
    subject_identifier=None,
    gender=FEMALE,
    age_in_years=40,
    ethnicity=BLACK,
    hiv_pos=YES,
    art_six_months=YES,
    on_rx_stable=YES,
    lives_nearby=YES,
    staying_nearby_6=YES,
    staying_nearby_12=YES,
    pregnant=NO,
    site=Site.objects.get_current(),
    part_two_report_datetime=get_utcnow() - relativedelta(days=1),
    congestive_heart_failure=NO,
    liver_disease=NO,
    alcoholism=NO,
    acute_metabolic_acidosis=NO,
    renal_function_condition=NO,
    tissue_hypoxia_condition=NO,
    acute_condition=NO,
    metformin_sensitivity=NO,
    advised_to_fast=YES,
    appt_datetime=get_utcnow() + relativedelta(days=1),
    part_three_report_datetime=get_utcnow(),
    weight=65,
    height=120,
    fasting=YES,
    fasting_duration_str="24:00",
    urine_bhcg_value=NO,
    hba1c_value=5.7,
    creatinine_value=0.6,
    creatinine_units=MILLIGRAMS_PER_DECILITER,
    fbg_value=6.9,
    fbg_units=MILLIMOLES_PER_LITER,
    fbg_datetime=get_utcnow(),
    ogtt_base_datetime=get_utcnow(),
    unsuitable_for_study=NO,
    unsuitable_agreed=NOT_APPLICABLE,
    vl_undetectable=YES,
)



from edc_model_admin.admin_site import EdcAdminSite

from .apps import AppConfig

meta_pharmacy_admin = EdcAdminSite(
    name="meta_pharmacy_admin",
    app_label=AppConfig.name,
    keep_delete_action=True,
)



from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, override_settings
from edc_action_item.models import ActionItem
from edc_adverse_event.constants import (
    AE_FOLLOWUP_ACTION,
    AE_TMG_ACTION,
    DEATH_REPORT_ACTION,
    DEATH_REPORT_TMG_ACTION,
)
from edc_constants.constants import CLOSED, NEW
from edc_reportable.constants import GRADE4, GRADE5
from model_bakery import baker

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin


class TestActions(MetaTestCaseMixin, TestCase):
    def test_ae_initial_creates_action(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        ae_initial = baker.make_recipe(
            "meta_ae.aeinitial", subject_identifier=subject_consent.subject_identifier
        )

        try:
            action_item = ActionItem.objects.get(
                action_identifier=ae_initial.action_identifier
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised.")
        else:
            self.assertEqual(action_item.status, CLOSED)
            self.assertEqual(
                action_item.subject_identifier, subject_consent.subject_identifier
            )

    def test_ae_initial_creates_ae_followup_action(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        ae_initial = baker.make_recipe(
            "meta_ae.aeinitial", subject_identifier=subject_consent.subject_identifier
        )

        action_item = ActionItem.objects.get(action_identifier=ae_initial.action_identifier)
        try:
            action_item = ActionItem.objects.get(
                parent_action_item=action_item,
                action_type__name=AE_FOLLOWUP_ACTION,
                subject_identifier=subject_consent.subject_identifier,
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised.")
        else:
            self.assertEqual(action_item.status, NEW)

    @override_settings(EDC_RANDOMIZATION_REGISTER_DEFAULT_RANDOMIZER=False)
    def test_ae_initial_G4_creates_ae_tmg_action(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        baker.make_recipe(
            "meta_ae.aeinitial",
            subject_identifier=subject_consent.subject_identifier,
            ae_grade=GRADE4,
        )

        try:
            ActionItem.objects.get(
                action_type__name=AE_TMG_ACTION,
                subject_identifier=subject_consent.subject_identifier,
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised.")

    def test_ae_initial_G5_creates_death_report_action(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        baker.make_recipe(
            "meta_ae.aeinitial",
            subject_identifier=subject_consent.subject_identifier,
            ae_grade=GRADE5,
        )
        try:
            ActionItem.objects.get(
                action_type__name=DEATH_REPORT_ACTION,
                subject_identifier=subject_consent.subject_identifier,
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised.")

        self.assertRaises(
            ObjectDoesNotExist,
            ActionItem.objects.get,
            action_type__name=DEATH_REPORT_TMG_ACTION,
            subject_identifier=subject_consent.subject_identifier,
        )

    def test_death_report_create_death_report_tmg_action(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        baker.make_recipe(
            "meta_ae.aeinitial",
            subject_identifier=subject_consent.subject_identifier,
            ae_grade=GRADE5,
        )
        action_item = ActionItem.objects.get(
            action_type__name=DEATH_REPORT_ACTION,
            subject_identifier=subject_consent.subject_identifier,
        )

        baker.make_recipe(
            "meta_ae.deathreport",
            subject_identifier=subject_consent.subject_identifier,
            action_identifier=action_item.action_identifier,
        )

        action_item.refresh_from_db()
        self.assertEqual(action_item.status, CLOSED)

        try:
            action_item = ActionItem.objects.get(
                action_type__name=DEATH_REPORT_TMG_ACTION,
                subject_identifier=subject_consent.subject_identifier,
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised.")
        else:
            self.assertEqual(action_item.status, NEW)



# Generated by Django 4.0.5 on 2022-06-29 14:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0113_bloodresultsrft_egfr_percent_change_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="egfr_percent_change",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=10,
                null=True,
                verbose_name="Percent change from baseline",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="egfr_percent_change",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=10,
                null=True,
                verbose_name="Percent change from baseline",
            ),
        ),
    ]



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixins import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyNonCrfModelFormMixin
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from meta_rando.models import RandomizationList

from ..models import Rx, Substitutions


class SubstitutionsFormValidator(PrnFormValidatorMixin, FormValidator):
    pass


class SubstitutionsForm(
    SiteModelFormMixin,
    OffstudyNonCrfModelFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = SubstitutionsFormValidator

    def clean(self):
        if self.cleaned_data.get("sid"):
            try:
                obj = Rx.objects.get(rando_sid=self.cleaned_data.get("sid"))
            except ObjectDoesNotExist:
                raise forms.ValidationError({"sid": "Unknown SID for this trial"})
            else:
                if obj.site.id != self.site.id:
                    raise forms.ValidationError(
                        {
                            "sid": (
                                "Unknown SID for this trial site. "
                                f"Expected SITE {obj.site.id}."
                            )
                        }
                    )
        if self.cleaned_data.get("dispensed_sid"):
            try:
                RandomizationList.objects.get(
                    sid=self.cleaned_data.get("dispensed_sid"),
                )
            except ObjectDoesNotExist:
                raise forms.ValidationError({"dispensed_sid": "Invalid SID for this trial"})

    class Meta:
        model = Substitutions
        fields = "__all__"



from django.contrib.sites.models import Site
from django.core.validators import RegexValidator
from django.db import models
from django_crypto_fields.fields import EncryptedCharField
from edc_constants.constants import PENDING, QUESTION_RETIRED
from edc_model.models import BaseUuidModel
from edc_screening.model_mixins import ScreeningModelMixin
from edc_screening.screening_identifier import (
    ScreeningIdentifier as BaseScreeningIdentifier,
)
from edc_utils import get_utcnow

from meta_consent.consents import consent_v1

from ..model_mixins import (
    CalculatedModelMixin,
    EligibilityModelMixin,
    PartOneFieldsModelMixin,
    PartThreeFieldsModelMixin,
    PartTwoFieldsModelMixin,
)


class SubjectScreeningModelError(Exception):
    pass


class ScreeningIdentifier(BaseScreeningIdentifier):
    template = "S{random_string}"


class SubjectScreening(
    PartOneFieldsModelMixin,
    PartTwoFieldsModelMixin,
    PartThreeFieldsModelMixin,
    EligibilityModelMixin,
    CalculatedModelMixin,
    ScreeningModelMixin,
    BaseUuidModel,
):
    identifier_cls = ScreeningIdentifier
    consent_definitions = [consent_v1]

    contact_number = EncryptedCharField(
        validators=[RegexValidator(r"^[0-9\-\(\)\ ]+$", message="Enter a valid number")],
        null=True,
        help_text="Provide a contact number if repeating glucose measures (Encryption: RSA)",
    )

    site = models.ForeignKey(Site, on_delete=models.PROTECT, null=True, related_name="+")

    def save(self, *args, **kwargs):
        if self._meta.label_lower == "meta_screening.subjectscreening":
            raise SubjectScreeningModelError("Unable to save. Save via P1-3 proxy models.")
        self.consent_ability = QUESTION_RETIRED
        super().save(*args, **kwargs)

    @property
    def human_readable_identifier(self):
        """Returns a humanized screening identifier."""
        x = self.screening_identifier
        return f"{x[0:4]}-{x[4:]}"

    @property
    def repeat_due_in_days(self):
        if self.repeat_glucose_performed == PENDING and self.ogtt_datetime:
            return (get_utcnow() - self.ogtt_datetime).days
        return 0

    class Meta:
        verbose_name = "Subject Screening"
        verbose_name_plural = "Subject Screening"



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
    select v.subject_identifier, fbg_value, fbg_datetime, null as `ogtt_value`, null as `ogtt_datetime`,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as `fasted`,
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucosefbg as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
    UNION
    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,
    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as `fasted`,
    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime
    from meta_subject_glucose as fbg
    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id
    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier
    """  # noqa
    sql_view = SqlViewGenerator(
        report_model="meta_reports.glucose_summary_view",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from django.db import models
from django_pandas.managers import DataFrameManager
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions


class LastImpRefill(QaReportModelMixin, models.Model):

    reference_date = models.DateField(null=True)

    imp_visit_code = models.FloatField(null=True)

    imp_visit_date = models.DateField(null=True)

    next_visit_code = models.FloatField(null=True)

    next_appt_date = models.DateField(null=True)

    visit_code = models.CharField(max_length=15, null=True)

    visit_code_sequence = models.IntegerField(null=True)

    days_since = models.IntegerField(null=True)

    days_until = models.IntegerField(null=True)

    objects = DataFrameManager()

    def recreate_db_view(self, **kwargs):
        raise NotImplementedError()

    class Meta(QaReportModelMixin.Meta):
        verbose_name = "Last IMP Refill"
        verbose_name_plural = "Last IMP Refills"
        default_permissions = qa_reports_permissions



from django.apps import apps as django_apps
from django.contrib.sites.managers import CurrentSiteManager
from django.db import models
from django.utils.translation import gettext_lazy as _
from edc_consent.field_mixins import (
    CitizenFieldsMixin,
    IdentityFieldsMixin,
    PersonalFieldsMixin,
    ReviewFieldsMixin,
    SampleCollectionFieldsMixin,
    VulnerabilityFieldsMixin,
)
from edc_consent.managers import ConsentObjectsManager
from edc_consent.model_mixins import ConsentModelMixin
from edc_constants.choices import YES_NO
from edc_constants.constants import NO, NOT_APPLICABLE
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierModelMixin
from edc_identifier.subject_identifier import SubjectIdentifier as BaseSubjectIdentifier
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_registration.model_mixins import UpdatesOrCreatesRegistrationModelMixin
from edc_sites.model_mixins import SiteModelMixin

from meta_pharmacy.constants import METFORMIN

from .model_mixins import SearchSlugModelMixin


class SubjectIdentifier(BaseSubjectIdentifier):
    template = "{protocol_number}-{site_id}-{sequence}"
    padding = 4


class SubjectConsent(
    ConsentModelMixin,
    SiteModelMixin,
    UpdatesOrCreatesRegistrationModelMixin,
    NonUniqueSubjectIdentifierModelMixin,
    IdentityFieldsMixin,
    ReviewFieldsMixin,
    PersonalFieldsMixin,
    SampleCollectionFieldsMixin,
    CitizenFieldsMixin,
    VulnerabilityFieldsMixin,
    SearchSlugModelMixin,
    BaseUuidModel,
):
    """A model completed by the user that captures the ICF."""

    subject_identifier_cls = SubjectIdentifier

    subject_screening_model = "meta_screening.subjectscreening"

    screening_identifier = models.CharField(
        verbose_name=_("Screening identifier"), max_length=50, unique=True
    )

    screening_datetime = models.DateTimeField(
        verbose_name=_("Screening datetime"), null=True, editable=False
    )

    ethnicity = models.CharField(
        max_length=15,
        help_text=_("from screening"),
        editable=False,
        null=True,
    )

    completed_by_next_of_kin = models.CharField(
        max_length=10, default=NO, choices=YES_NO, editable=False
    )

    study_medication_name = models.CharField(
        max_length=25,
        default=METFORMIN,
        editable=False,
        help_text="Used by `create_prescription` in the signal",
    )

    on_site = CurrentSiteManager()

    objects = ConsentObjectsManager()

    history = HistoricalRecords()

    def __str__(self):
        return f"{self.subject_identifier} V{self.version}"

    def save(self, *args, **kwargs):
        subject_screening = self.get_subject_screening()
        self.screening_datetime = subject_screening.report_datetime
        self.subject_type = "subject"
        self.citizen = NOT_APPLICABLE
        self.ethnicity = subject_screening.ethnicity
        super().save(*args, **kwargs)

    def natural_key(self):
        return self.subject_identifier, self.version

    def get_subject_screening(self):
        """Returns the subject screening model instance.

        Instance must exist since SubjectScreening is completed
        before consent.
        """
        model_cls = django_apps.get_model(self.subject_screening_model)
        return model_cls.objects.get(screening_identifier=self.screening_identifier)

    @property
    def registration_unique_field(self):
        """Required for UpdatesOrCreatesRegistrationModelMixin."""
        return "subject_identifier"

    class Meta(ConsentModelMixin.Meta, BaseUuidModel.Meta):
        unique_together = (
            ("subject_identifier", "version"),
            ("subject_identifier", "screening_identifier"),
            ("first_name", "dob", "initials", "version"),
        )



from edc_adverse_event.view_mixins import DeathReportListboardViewMixin
from reportlab.lib.units import cm

from meta_reports.death_report import DeathPdfReport


class CustomDeathReport(DeathPdfReport):
    logo_data = {
        "app_label": "meta_edc",
        "filename": "meta_logo.png",
        "first_page": (4.0 * cm, 0.83 * cm),
        "later_pages": (3.0 * cm, 0.625 * cm),
    }


class DeathReportListboardView(DeathReportListboardViewMixin):
    pdf_report_cls = CustomDeathReport



from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import post_save
from django.dispatch import receiver
from edc_constants.constants import YES
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from meta_subject.models import SubjectVisit, UrinePregnancy
from meta_visit_schedule.constants import (
    SCHEDULE,
    SCHEDULE_DM_REFERRAL,
    SCHEDULE_PREGNANCY,
    VISIT_SCHEDULE,
)

from . import OffScheduleDmReferral
from .dm_referral import DmReferral
from .offschedule import OffSchedule
from .pregnancy_notification import PregnancyNotification


@receiver(
    post_save,
    weak=False,
    sender=PregnancyNotification,
    dispatch_uid="update_schedule_on_pregnancy_notification_post_save",
)
def update_schedule_on_pregnancy_notification_post_save(sender, instance, raw, **kwargs):
    if not raw:
        try:
            OffSchedule.objects.get(subject_identifier=instance.subject_identifier)
        except ObjectDoesNotExist:
            last_subject_visit = (
                SubjectVisit.objects.filter(
                    subject_identifier=instance.subject_identifier,
                    schedule_name=SCHEDULE,
                )
                .order_by("report_datetime")
                .last()
            )
            visit_schedule = site_visit_schedules.get_visit_schedule(
                visit_schedule_name=VISIT_SCHEDULE
            )
            schedule = visit_schedule.schedules.get(SCHEDULE)
            schedule.take_off_schedule(
                instance.subject_identifier, last_subject_visit.report_datetime
            )
            schedule = visit_schedule.schedules.get(SCHEDULE_PREGNANCY)
            schedule.put_on_schedule(
                onschedule_datetime=last_subject_visit.report_datetime,
                subject_identifier=instance.subject_identifier,
            )


@receiver(
    post_save,
    weak=False,
    sender=PregnancyNotification,
    dispatch_uid="update_urine_pregnancy_on_pregnancy_notification_on_post_save",
)
def update_urine_pregnancy_on_pregnancy_notification_on_post_save(
    sender, instance, raw, **kwargs
):
    if not raw:
        if instance.bhcg_confirmed == YES:
            UrinePregnancy.objects.filter(
                subject_visit__subject_identifier=instance.subject_identifier,
                notified=False,
                assay_date__lte=instance.report_datetime.date(),
            ).update(
                notified_datetime=instance.report_datetime,
                notified=True,
            )


@receiver(
    post_save,
    weak=False,
    sender=DmReferral,
    dispatch_uid="update_schedule_on_dm_referral_post_save",
)
def update_schedule_on_dm_referral_post_save(sender, instance, raw, **kwargs):
    if not raw:
        try:
            OffScheduleDmReferral.objects.get(subject_identifier=instance.subject_identifier)
        except ObjectDoesNotExist:
            last_subject_visit = (
                SubjectVisit.objects.filter(
                    subject_identifier=instance.subject_identifier,
                    schedule_name=SCHEDULE,
                )
                .order_by("report_datetime")
                .last()
            )
            visit_schedule = site_visit_schedules.get_visit_schedule(
                visit_schedule_name=VISIT_SCHEDULE
            )
            schedule = visit_schedule.schedules.get(SCHEDULE)
            schedule.take_off_schedule(
                instance.subject_identifier, last_subject_visit.report_datetime
            )
            schedule = visit_schedule.schedules.get(SCHEDULE_DM_REFERRAL)
            schedule.put_on_schedule(
                onschedule_datetime=last_subject_visit.report_datetime,
                subject_identifier=instance.subject_identifier,
            )



# Generated by Django 4.2.10 on 2024-02-22 15:16

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0002_alter_domain_unique"),
        ("meta_subject", "0176_alter_glucosefbg_glucose_value_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alp_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALP",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALT",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alp_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALP",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALT",
            ),
        ),
    ]



import numpy as np
import pandas as pd
from django.apps import apps as django_apps
from edc_constants.constants import NO, YES
from edc_pdutils.dataframes import (
    get_crf,
    get_eos,
    get_subject_consent,
    get_subject_visit,
)
from edc_utils import get_utcnow

from ..constants import (
    CASE_EOS,
    CASE_FBG_ONLY,
    CASE_FBGS_WITH_FIRST_OGTT,
    CASE_FBGS_WITH_SECOND_OGTT,
    CASE_OGTT,
    endpoint_cases,
    endpoint_columns,
)
from ..utils import (
    get_empty_endpoint_df,
    get_test_string,
    get_unique_subject_identifiers,
    get_unique_visit_codes,
)
from .endpoint_by_date import EndpointByDate


def calculate_fasting_hrs(df: pd.DataFrame):
    df.loc[(df["fasting"] == NO), "fasting_duration_delta"] = pd.NaT
    if df.empty:
        df["fasting_hrs"] = np.nan
    else:
        df["fasting_hrs"] = df["fasting_duration_delta"].apply(
            lambda s: np.nan if pd.isna(s) else s.total_seconds() / 3600
        )
    return df


class GlucoseEndpointsByDate:
    """
    Usage:
        cls = GlucoseEndpointsByDate()
        cls.run()

        # subjects who reached endpoint
        cls.endpoint_only_df.endpoint_type.value_counts()
        cls.endpoint_only_df.endpoint_label.value_counts()

        # subjects who reached endpoint with total
        result_df = cls.endpoint_only_df.endpoint_label.value_counts().to_frame().reset_index()
        result_df.columns = ["endpoint_label", "total"]
        result_df.loc[-1] = ["total", result_df.total.sum()]
        result_df = result_df.reset_index(drop=True)
        result_df
    """

    fbg_threshhold = 7.0
    ogtt_threshhold = 11.1
    endpoint_cls = EndpointByDate
    keep_cols = [
        "fasting",
        "fasting_hrs",
        "fbg_value",
        "fbg_units",
        "fbg_datetime",
        "ogtt_value",
        "ogtt_units",
        "ogtt_datetime",
        "source",
        "report_datetime",
        "subject_visit_id",
        "subject_identifier",
        "visit_code",
        "visit_datetime",
        "site_id",
        "baseline_datetime",
    ]

    def __init__(
        self, subject_identifiers: list[str] | None = None, case_list: list[int] | None = None
    ):
        self._glucose_fbg_df = pd.DataFrame()
        self._glucose_fbg_ogtt_df = pd.DataFrame()
        self.endpoint_only_df = pd.DataFrame()

        self.subject_identifiers = subject_identifiers or []
        self.case_list = case_list or [
            CASE_OGTT,
            CASE_FBGS_WITH_FIRST_OGTT,
            CASE_FBGS_WITH_SECOND_OGTT,
            CASE_EOS,
        ]
        self.endpoint_cases = {k: v for k, v in endpoint_cases.items() if k in self.case_list}

        # merge two model DFs
        if self.glucose_fbg_ogtt_df.empty:
            self.df = self.glucose_fbg_df.copy()
            self.df[["ogtt_value", "ogtt_units"]] = np.nan
            self.df[["ogtt_datetime"]] = pd.NaT
        elif self.glucose_fbg_df.empty:
            self.df = self.glucose_fbg_ogtt_df.copy()
        else:
            self.df = self.glucose_fbg_ogtt_df.merge(
                self.glucose_fbg_df,
                on=["subject_visit_id"],
                how="outer",
                indicator=True,
                suffixes=("", "_y"),
            )
        # cast as ...
        for col in ["fasting_hrs", "fbg_value"]:
            self.df[col] = self.df[col].astype("float64")
            if f"{col}_y" in self.df.columns:
                self.df[f"{col}_y"] = self.df[f"{col}_y"].astype("float64")
        for col in ["fasting", "fbg_units", "source"]:
            self.df[col] = self.df[col].astype("object")
            if f"{col}_y" in self.df.columns:
                self.df[f"{col}_y"] = self.df[f"{col}_y"].astype("object")
        self.df = self.df.drop(
            columns=[col for col in self.df.columns if col.endswith("_y") or col == "_merge"]
        )
        self.df = self.df.reset_index(drop=True)

        # merge w/ subject_visit
        subject_visit_df = get_subject_visit(
            "meta_subject.subjectvisit", subject_identifiers=self.subject_identifiers
        )
        self.df = subject_visit_df.merge(
            self.df, on=["subject_visit_id"], how="left", suffixes=("", "_y")
        )
        self.df = self.df[[col for col in self.keep_cols]]
        self.df = self.df.reset_index(drop=True)

        # pivot right_only cols
        cols = [
            "fasting",
            "fasting_hrs",
            "fbg_value",
            "fbg_units",
            "fbg_datetime",
            "source",
            "report_datetime",
        ]
        for col in cols:
            if f"{col}_y" in self.df.columns and not self.df[f"{col}_y"].isnull().all():
                self.df.loc[
                    (self.df["_merge"].isin(["both", "right_only"])) & (self.df[col].isna()),
                    col,
                ] = self.df[f"{col}_y"]
        # if fbg_datetime still null, use visit datetime
        if self.df["fbg_datetime"].isnull().all():
            self["fbg_datetime"] = self.df["visit_datetime"]
        else:
            self.df.loc[(self.df["fbg_datetime"].isna()), "fbg_datetime"] = self.df[
                "visit_datetime"
            ]
        self.df = self.df.drop(
            columns=[col for col in self.df.columns if col.endswith("_y") or col == "_merge"]
        )
        self.df = self.df.reset_index(drop=True)

        self.merge_with_consent()
        self.merge_with_eos()

        self.add_calculated_days_from_baseline_to_event_columns()

        # label rows by type of glu tests (ones with value)
        self.df["test"] = self.df.apply(get_test_string, axis=1)
        self.df = self.df.reset_index(drop=True)

        self.visit_codes_df = get_unique_visit_codes(self.df)
        self.subject_identifiers_df = get_unique_subject_identifiers(self.df)

        self.df = self.df.sort_values(by=["subject_identifier", "fbg_datetime"])
        self.df = self.df.reset_index(drop=True)

        self.working_df = self.df.copy()
        self.working_df["endpoint"] = 0
        self.endpoint_df = get_empty_endpoint_df()

    def run(self):
        self.pre_check_endpoint()
        for index, row in self.subject_identifiers_df.iterrows():
            subject_df = self.get_subject_df(row["subject_identifier"])
            subject_df = self.check_endpoint_by_fbg_for_subject(
                subject_df, case_list=[CASE_FBGS_WITH_FIRST_OGTT, CASE_FBGS_WITH_SECOND_OGTT]
            )
            if len(subject_df.loc[subject_df["endpoint"] == 1]) == 1:
                self.append_subject_to_endpoint_df(subject_df)
                self.remove_subject_from_working_df(row)

        if CASE_FBG_ONLY in self.endpoint_cases:
            for index, row in self.subject_identifiers_df.iterrows():
                subject_df = self.get_subject_df(row["subject_identifier"])
                subject_df = self.check_endpoint_by_fbg_for_subject(
                    subject_df, case_list=[CASE_FBG_ONLY]
                )
                if len(subject_df.loc[subject_df["endpoint"] == 1]) == 1:
                    self.append_subject_to_endpoint_df(subject_df)
                    self.remove_subject_from_working_df(row)

        self.post_check_endpoint()
        self.merge_with_final_endpoints()

    @property
    def glucose_fbg_df(self) -> pd.DataFrame:
        """Returns a prepared Dataframe of CRF
        meta_subject.glucosefbg.

        Note: meta_subject.glucosefbg has only FBG measures.
        """
        if self._glucose_fbg_df.empty:
            df = get_crf(
                model="meta_subject.glucosefbg",
                subject_identifiers=self.subject_identifiers,
                # subject_visit_model="meta_subject.subjectvisit",
            )
            df["source"] = "meta_subject.glucosefbg"
            df.rename(columns={"fbg_fasting": "fasting"}, inplace=True)
            df.loc[(df["fasting"] == "fasting"), "fasting"] = YES
            df.loc[(df["fasting"] == "non_fasting"), "fasting"] = NO
            df = calculate_fasting_hrs(df)
            # df = df[[col for col in self.keep_cols if not col.startswith("ogtt")]]
            df = df.reset_index(drop=True)
            self._glucose_fbg_df = df
        return self._glucose_fbg_df

    @property
    def glucose_fbg_ogtt_df(self):
        """Returns a prepared Dataframe of CRF meta_subject.glucose.

        Note: meta_subject.glucose has FBG and OGTT measures.
        """
        if self._glucose_fbg_ogtt_df.empty:
            df = get_crf(
                model="meta_subject.glucose",
                subject_identifiers=self.subject_identifiers,
                # subject_visit_model="meta_subject.subjectvisit",
            )
            df["source"] = "meta_subject.glucose"
            df = calculate_fasting_hrs(df)
            # df = df[self.keep_cols]
            df = df.reset_index(drop=True)
            self._glucose_fbg_ogtt_df = df
        return self._glucose_fbg_ogtt_df

    def merge_with_consent(self):
        """Merge in consent DF."""
        df_consent = get_subject_consent(
            "meta_consent.subjectconsent", subject_identifiers=self.subject_identifiers
        )
        self.df = pd.merge(
            self.df, df_consent, on="subject_identifier", how="left", suffixes=("", "_y")
        )
        self.df = self.df.sort_values(by=["subject_identifier", "fbg_datetime"])
        self.df = self.df.reset_index(drop=True)

    def merge_with_eos(self):
        """Merge in EoS DF.

        Drops patients who were taken off study by late exclusion.
        """
        df_eos = get_eos("meta_prn.endofstudy", subject_identifiers=self.subject_identifiers)
        df_eos = df_eos[
            df_eos["offstudy_reason"]
            != (
                "Patient fulfilled late exclusion criteria (due to abnormal blood "
                "values or raised blood pressure at enrolment"
            )
        ]
        self.df = pd.merge(
            self.df, df_eos, on="subject_identifier", how="left", suffixes=("", "_y")
        )
        self.df = self.df.sort_values(by=["subject_identifier", "fbg_datetime"])
        self.df = self.df.reset_index(drop=True)

    def add_calculated_days_from_baseline_to_event_columns(self):
        """Add columns that calculate number of days from
        baseline to visit, fbg, and ogtt.
        """
        self.df["visit_days"] = np.nan
        self.df["fbg_days"] = np.nan
        self.df["ogtt_days"] = np.nan
        self.df["test"] = np.nan
        self.df["visit_days"] = (
            self.df["visit_datetime"] - self.df["baseline_datetime"]
        ).dt.days
        if not self.df["fbg_datetime"].isnull().all():
            self.df["fbg_days"] = (
                self.df["fbg_datetime"] - self.df["baseline_datetime"]
            ).dt.days
        if not self.df["ogtt_datetime"].isnull().all():
            self.df["ogtt_days"] = (
                self.df["ogtt_datetime"] - self.df["baseline_datetime"]
            ).dt.days
        self.df["visit_days"] = pd.to_numeric(self.df["visit_days"], downcast="integer")
        self.df["fbg_days"] = pd.to_numeric(self.df["fbg_days"], downcast="integer")
        self.df["ogtt_days"] = pd.to_numeric(self.df["ogtt_days"], downcast="integer")
        self.df = self.df.reset_index(drop=True)

    def pre_check_endpoint(self):
        """Flag subjects that met endpoint by hitting the OGTT
        threshold.

        Add them to the endpoint_df and remove them from the
        working_df.

        Subject must have fasted at the timepoint.

        The OGTT must have an FBG measure at the same timepoint.
        The value of the FBG is not considered.

        Most of these where taken off study for the OGTT. We are
        using the OGTT as the reason/date instead of the offstudy
        reason/date.

        See `merge_with_final_endpoints` where we pick the date of
        the first OGTT.
        """
        subject_endpoint_df = self.working_df.loc[
            (self.working_df["ogtt_value"] >= self.ogtt_threshhold)
            & (self.working_df["fasting"] == YES)
            & (self.working_df["fbg_value"].notna())
        ].copy()
        subject_endpoint_df.sort_values(by=["subject_identifier", "fbg_datetime"])
        subject_endpoint_df = subject_endpoint_df.reset_index(drop=True)
        subject_endpoint_df = subject_endpoint_df.drop_duplicates(
            subset=["subject_identifier"], keep="first"
        )
        subject_endpoint_df = subject_endpoint_df.reset_index(drop=True)
        if not subject_endpoint_df.empty:
            # flag the selected endpoint rows as endpoints
            subject_endpoint_df["endpoint"] = 1
            subject_endpoint_df["endpoint_label"] = self.endpoint_cases[CASE_OGTT]
            subject_endpoint_df["endpoint_type"] = CASE_OGTT
            subject_endpoint_df["interval_in_days"] = np.nan

            # add back the others rows for these subjects
            subjects_df = self.working_df.loc[
                (
                    self.working_df["subject_identifier"].isin(
                        subject_endpoint_df["subject_identifier"]
                    )
                    & ~(
                        self.working_df["fbg_datetime"].isin(
                            subject_endpoint_df["fbg_datetime"]
                        )
                    )
                )
            ].copy()
            subjects_df = subjects_df.reset_index(drop=True)
            subjects_df["endpoint"] = np.nan
            subjects_df["endpoint_label"] = None
            subjects_df["endpoint_type"] = None
            subjects_df["interval_in_days"] = np.nan
            subjects_df = pd.concat([subjects_df, subject_endpoint_df])
            subjects_df = subjects_df.reset_index(drop=True)

            self.append_subject_to_endpoint_df(subjects_df[endpoint_columns])
            self.remove_subjects_from_working_df(subjects_df)

    def append_subject_to_endpoint_df(self, subject_df: pd.DataFrame) -> None:
        """Appends all rows of a subject, or subjects, to the
        Endpoints DF.
        """
        if self.endpoint_df.empty:
            self.endpoint_df = subject_df.copy()
        else:
            self.endpoint_df = pd.concat([self.endpoint_df, subject_df])
            self.endpoint_df = self.endpoint_df.sort_values(
                by=["subject_identifier", "visit_code"]
            )
            self.endpoint_df = self.endpoint_df.reset_index(drop=True)

    def remove_subject_from_working_df(self, row: pd.Series) -> None:
        """Removes one subject from the working DF given a Series with
        value `subject_identifier`.
        """
        self.working_df = self.working_df.drop(
            index=self.working_df[
                self.working_df["subject_identifier"] == row["subject_identifier"]
            ].index
        )
        self.working_df = self.working_df.reset_index(drop=True)

    def remove_subjects_from_working_df(self, rows: pd.DataFrame) -> None:
        """Removes subjects from the working DF given a DF with
        column `subject_identifier`.
        """
        self.working_df = self.working_df.drop(
            index=self.working_df.loc[
                self.working_df["subject_identifier"].isin(rows["subject_identifier"])
            ].index
        )
        self.working_df = self.working_df.reset_index(drop=True)

    def get_subject_df(self, subject_identifier: str) -> pd.DataFrame:
        subject_df = self.working_df.loc[
            self.working_df["subject_identifier"] == subject_identifier
        ].copy()
        subject_df["interval_in_days"] = np.nan
        subject_df["endpoint_type"] = None
        subject_df["endpoint_label"] = None
        subject_df["endpoint"] = 0
        subject_df = subject_df[endpoint_columns]
        subject_df = subject_df.sort_values(["subject_identifier", "fbg_datetime"])
        subject_df = subject_df.reset_index(drop=True)
        return subject_df

    def check_endpoint_by_fbg_for_subject(
        self, subject_df: pd.DataFrame, case_list: list[int] | None = None
    ) -> pd.DataFrame:
        case_list = case_list or [2, 3]
        endpoint = self.endpoint_cls(
            subject_df=subject_df,
            fbg_threshhold=self.fbg_threshhold,
            ogtt_threshhold=self.ogtt_threshhold,
            case_list=case_list,
        )
        return endpoint.subject_df

    def post_check_endpoint(self):
        df_eos = self.working_df.loc[
            self.working_df["offstudy_reason"] == "Patient developed diabetes"
        ].copy()
        df_eos["endpoint"] = 1
        df_eos["endpoint_label"] = self.endpoint_cases[CASE_EOS]
        df_eos["endpoint_type"] = CASE_EOS
        df_eos["interval_in_days"] = np.nan
        df_eos = df_eos.reset_index(drop=True)
        self.append_subject_to_endpoint_df(df_eos[endpoint_columns])
        self.working_df = self.working_df.drop(
            index=self.working_df.loc[
                self.working_df["subject_identifier"].isin(df_eos["subject_identifier"])
            ].index
        )

    def merge_with_final_endpoints(self):
        """Merge endpoint_df with original df"""
        if self.endpoint_df.empty:
            self.df = self.df[~(self.df["subject_identifier"].isin(self.subject_identifiers))]
        else:
            self.endpoint_df["test"] = self.endpoint_df.apply(get_test_string, axis=1)
            self.endpoint_df.loc[self.endpoint_df["endpoint"] == 1, "days_to_endpoint"] = (
                self.endpoint_df["fbg_datetime"] - self.endpoint_df["baseline_datetime"]
            ).dt.days

            # Create DF of subjects taken offstudy (EOS) where endpoint==1.
            # Keep the last record for the subject by fbg_datetime.
            df1 = self.endpoint_df.copy()
            df1 = df1[
                (df1["endpoint_type"].isin([CASE_EOS, CASE_OGTT])) & (df1["endpoint"] == 1)
            ]
            df1 = df1.sort_values(["subject_identifier", "fbg_datetime"])
            df1 = df1.reset_index(drop=True)
            df1 = df1.set_index(["subject_identifier"])
            df1 = df1[~df1.index.duplicated(keep="last")]
            df1 = df1.reset_index(drop=False)

            # Create DF of subjects still on-study where endpoint==1.
            # Keep the first record for the subject by fbg_datetime.
            df2 = self.endpoint_df.copy()
            df2 = df2[
                ~(df2["endpoint_type"].isin([CASE_EOS, CASE_OGTT])) & (df2["endpoint"] == 1)
            ]
            df2 = df2.sort_values(["subject_identifier", "fbg_datetime"])
            df2 = df2.reset_index(drop=True)
            df2 = df2.set_index(["subject_identifier"])
            df2 = df2[~df2.index.duplicated(keep="first")]
            df2 = df2.reset_index(drop=False)

            # create new DF with ONE row per subject for those that reached
            # the endpoint (endpoint=1) by merging two DFs above.
            self.endpoint_only_df = pd.concat([df1, df2])
            self.endpoint_only_df = self.endpoint_only_df.reset_index(drop=True)

            self.df = pd.merge(
                self.df,
                self.endpoint_only_df[["subject_identifier", "visit_code", "endpoint"]],
                on=["subject_identifier", "visit_code"],
                how="left",
                suffixes=("", "_y"),
            )
        self.df = self.df.sort_values(by=["subject_identifier", "fbg_datetime"])
        self.df = self.df.reset_index(drop=True)

    def to_model(self):
        """Write endpoint_only_df to the Endpoints model"""
        df = self.endpoint_only_df
        model = "meta_reports.endpoints"
        now = get_utcnow()
        model_cls = django_apps.get_model(model)
        if self.subject_identifiers:
            model_cls.objects.filter(subject_identifier__in=self.subject_identifiers).delete()
        else:
            model_cls.objects.all().delete()
        created = 0
        if not df.empty:
            df["fbg_datetime"] = df["fbg_datetime"].dt.tz_localize("UTC")
            df["baseline_datetime"] = df["baseline_datetime"].dt.tz_localize("UTC")
            data = [
                model_cls(
                    subject_identifier=row["subject_identifier"],
                    site_id=row["site_id"],
                    baseline_date=(
                        None if pd.isna(row["baseline_datetime"]) else row["baseline_datetime"]
                    ),
                    visit_code=None if pd.isna(row["visit_code"]) else row["visit_code"],
                    fbg_value=(None if pd.isna(row["fbg_value"]) else row["fbg_value"]),
                    ogtt_value=None if pd.isna(row["ogtt_value"]) else row["ogtt_value"],
                    fbg_date=(None if pd.isna(row["fbg_datetime"]) else row["fbg_datetime"]),
                    fasting=(None if pd.isna(row["fasting"]) else row["fasting"]),
                    endpoint_label=(
                        None if pd.isna(row["endpoint_label"]) else row["endpoint_label"]
                    ),
                    offstudy_date=(
                        None if pd.isna(row["offstudy_datetime"]) else row["offstudy_datetime"]
                    ),
                    offstudy_reason=(
                        None if pd.isna(row["offstudy_reason"]) else row["offstudy_reason"]
                    ),
                    report_model=model,
                    created=now,
                )
                for _, row in df.iterrows()
            ]
            created = len(model_cls.objects.bulk_create(data))
        return created



# Generated by Django 5.0.4 on 2024-04-09 04:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0183_alter_dmfollowup_on_dm_medications_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="glucose",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "verbose_name": "Glucose (FBG/RBG, OGTT)",
                "verbose_name_plural": "Glucose (FBG/RBG, OGTT)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalglucose",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Glucose (FBG/RBG, OGTT)",
                "verbose_name_plural": "historical Glucose (FBG/RBG, OGTT)",
            },
        ),
    ]



# Generated by Django 5.1 on 2024-09-11 00:52
from django.apps import apps as django_apps
from django.db import migrations


def update_report_datetime_as_none(apps, schema_editor):
    model_cls = django_apps.get_model("meta_pharmacy.substitutions")
    model_cls.objects.update(report_datetime=None)


class Migration(migrations.Migration):

    dependencies = [
        ("meta_pharmacy", "0004_alter_historicalsubstitutions_report_datetime_and_more"),
    ]

    operations = [migrations.RunPython(update_report_datetime_as_none)]



from textwrap import fill

from edc_pdf_reports.crf_pdf_report import CrfPdfReport
from edc_randomization.auth_objects import RANDO_UNBLINDED
from edc_randomization.models import RandomizationList
from reportlab.lib.units import cm
from reportlab.platypus import Table

from meta_subject.models import FollowupVitals


def get_weight_for_timepoint(subject_identifier=None, reference_dt=None):
    qs = FollowupVitals.objects.filter(
        subject_visit__appointment__subject_identifier=subject_identifier,
        report_datetime__lte=reference_dt,
    ).order_by("-report_datetime")
    if qs:
        return qs[0].weight
    return None


class MetaCrfPdfReport(CrfPdfReport):
    logo_data = {
        "app_label": "meta_edc",
        "filename": "meta_logo.png",
        "first_page": (4.0 * cm, 0.83 * cm),
        "later_pages": (3.0 * cm, 0.625 * cm),
    }

    def __init__(self, subject_identifier=None, **kwargs):
        super().__init__(**kwargs)
        self.subject_identifier = subject_identifier
        self.drug_assignment = RandomizationList.objects.get(
            subject_identifier=self.subject_identifier
        ).get_drug_assignment_display()

    def draw_demographics(self, story, **kwargs):
        model_obj = getattr(self, self.model_attr)
        weight = get_weight_for_timepoint(
            subject_identifier=self.subject_identifier,
            reference_dt=model_obj.report_datetime,
        )
        assignment = "*****************"
        if self.request.user.groups.filter(name=RANDO_UNBLINDED).exists():
            assignment = fill(self.drug_assignment, width=80)
        rows = [
            ["Subject:", self.subject_identifier],
            [
                "Gender/Age:",
                f"{self.registered_subject.get_gender_display()} {self.age}",
            ],
            ["Weight:", f"{weight} kg"],
            [
                "Study site:",
                f"{self.registered_subject.site.id}: "
                f"{self.registered_subject.site.name.title()}",
            ],
            [
                "Randomization date:",
                self.registered_subject.randomization_datetime.strftime("%Y-%m-%d %H:%M"),
            ],
            ["Assignment:", assignment],
        ]

        t = Table(rows, (4 * cm, 14 * cm))
        self.set_table_style(t, bg_cmd=self.bg_cmd)
        t.hAlign = "LEFT"
        story.append(t)



# Generated by Django 3.2.13 on 2022-09-18 02:08

from django.db import migrations, models
import edc_crf.model_mixins.crf_status_model_mixin


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0150_auto_20220914_0039"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="healtheconomicssimple",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics (M3)",
                "verbose_name_plural": "Health Economics (M3)",
            },
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsglu",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsglu",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultshba1c",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultshba1c",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsins",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsins",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultshba1c",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultshba1c",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsins",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsins",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectvisitmissed",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectvisitmissed",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="subjectvisitmissed",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="subjectvisitmissed",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AlterField(
            model_name="complications",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="complicationsglycemia",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="concomitantmedication",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="delivery",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="dietandlifestyle",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="egfrdropnotification",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="eq5d3l",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="followupvitals",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomics",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicssimple",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcomplications",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcomplicationsglycemia",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconcomitantmedication",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldietandlifestyle",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalegfrdropnotification",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomics",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicssimple",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmalariatest",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedicationadherence",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalotherarvregimens",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatienthistory",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancyupdate",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsf12",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalurinedipsticktest",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalurinepregnancy",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="malariatest",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="medicationadherence",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="otherarvregimens",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="patienthistory",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancyupdate",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="sf12",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="studymedication",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="urinedipsticktest",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AlterField(
            model_name="urinepregnancy",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsfbc",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_54fe81_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsglu",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_93fda3_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultshba1c",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_c357be_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsins",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_7417a1_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslft",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_00da20_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslipid",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_9692a2_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsrft",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_b66983_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="complications",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_6b215d_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="complicationsglycemia",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_49362b_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="concomitantmedication",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_1275b5_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="delivery",
            index=models.Index(
                fields=["subject_visit", "site", "id"], name="meta_subjec_subject_e0559c_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="delivery",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_9ab861_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="dietandlifestyle",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_756ef7_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="egfrdropnotification",
            index=models.Index(
                fields=["subject_visit", "site", "id"], name="meta_subjec_subject_e1dd63_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="egfrdropnotification",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_a11e07_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="eq5d3l",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_51b806_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="followupexamination",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_38241c_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="followupvitals",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_389589_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="glucose",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_c23092_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="healtheconomics",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_f0fde8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="healtheconomicssimple",
            index=models.Index(
                fields=["subject_visit", "site", "id"], name="meta_subjec_subject_a13289_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="healtheconomicssimple",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_cad3f5_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="hepatitistest",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_9fa552_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="malariatest",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_c33a9b_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="medicationadherence",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_0fbe25_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="mnsi",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_24180c_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="otherarvregimens",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_a313e9_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="patienthistory",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_a7a068_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="physicalexam",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_d01a41_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="pregnancyupdate",
            index=models.Index(
                fields=["subject_visit", "site", "id"], name="meta_subjec_subject_4cce95_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pregnancyupdate",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_272949_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="sf12",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_12a9fb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="studymedication",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_d338e8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="subjectvisitmissed",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_a6b7ab_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="urinedipsticktest",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_d270e1_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="urinepregnancy",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="meta_subjec_subject_062bc3_idx",
            ),
        ),
    ]
